------------
-- tables.sql
--
--      Create Veil demo database tables.
--
--      Copyright (c) 2005, 2006 Marc Munro
--      Author:  Marc Munro
--	License: BSD
--
-- $Id: tables.sql,v 1.2 2006/10/04 15:50:09 bloodnok Exp $
------------
\echo - Creating tables...


\echo - ...privileges...
create table privileges (
    privilege_id	integer not null,
    privilege_name	varchar(80) not null
);
alter table privileges add constraint privilege__pk
    primary key(privilege_id);


\echo - ...role...
create table roles (
    role_id	integer not null,
    role_name	varchar(80) not null
);
alter table roles add constraint role__pk
    primary key(role_id);


\echo - ...role_privileges...
create table role_privileges (
    role_id		integer not null,
    privilege_id	integer not null
);

alter table role_privileges add constraint role_privilege__pk
    primary key(role_id, privilege_id);

alter table role_privileges add constraint role_privilege__role_fk
    foreign key(role_id)
    references roles(role_id);

alter table role_privileges add constraint role_privilege__priv_fk
    foreign key(privilege_id)
    references privileges(privilege_id);

\echo - ...role_roles...
create table role_roles (
    role_id		integer not null,
    has_role_id		integer not null
);

alter table role_roles add constraint role_role__pk
    primary key(role_id, has_role_id);

alter table role_roles add constraint role_role__role_fk
    foreign key(role_id)
    references roles(role_id);

alter table role_roles add constraint role_role__has_role_fk
    foreign key(has_role_id)
    references roles(role_id);


\echo - ...persons...
create sequence person_id_seq;
create table persons (
    person_id		integer not null,
    person_name		varchar(80) not null
);
alter table persons add constraint person__pk
    primary key(person_id);


\echo - ...projects...
create sequence project_id_seq;
create table projects (
    project_id		integer not null,
    project_name	varchar(80) not null
);
alter table projects add constraint project__pk
    primary key(project_id);


\echo - ...detail_types...
create table detail_types (
    detail_type_id	  integer not null,
    required_privilege_id integer not null,
    detail_type_name	  varchar(80) not null
);
alter table detail_types add constraint detail_type__pk
    primary key(detail_type_id);

alter table detail_types add constraint detail_type__priv_fk
    foreign key(required_privilege_id)
    references privileges(privilege_id);


\echo - ...assignments...
create table assignments (
    project_id		integer not null,
    person_id		integer not null,
    role_id		integer not null
);
alter table assignments add constraint assignment__pk
    primary key(project_id, person_id);

alter table assignments add constraint assignment__project_fk
    foreign key(project_id)
    references projects(project_id);

alter table assignments add constraint assignment__person_fk
    foreign key(person_id)
    references persons(person_id);

alter table assignments add constraint assignment__role_fk
    foreign key(role_id)
    references roles(role_id);


\echo - ...person_roles...
create table person_roles (
    person_id		integer not null,
    role_id		integer not null
);
alter table person_roles add constraint person_role__pk
    primary key(person_id, role_id);

alter table person_roles add constraint person_role__person_fk
    foreign key(person_id)
    references persons(person_id);

alter table person_roles add constraint person_role__role_fk
    foreign key(role_id)
    references roles(role_id);


\echo - ...project_details...
create table project_details (
    project_id		integer not null,
    detail_type_id	integer not null,
    value		text not null
);
alter table project_details add constraint project_detail__pk
    primary key(project_id, detail_type_id);

alter table project_details add constraint project_detail__project_fk
    foreign key(project_id)
    references projects(project_id);

alter table project_details add constraint project_detail__detail_fk
    foreign key(detail_type_id)
    references detail_types(detail_type_id);


\echo - ...person_details...
create table person_details (
    person_id		integer not null,
    detail_type_id	integer not null,
    value		text not null
);
alter table person_details add constraint person_detail__pk
    primary key(person_id, detail_type_id);

alter table person_details add constraint person_detail__person_fk
    foreign key(person_id)
    references persons(person_id);

alter table person_details add constraint person_detail__detail_fk
    foreign key(detail_type_id)
    references detail_types(detail_type_id);

