------------
-- views.sql
--
--      Create secured views for Veil demo database.
--
--      Copyright (c) 2005, 2006 Marc Munro
--      Author:  Marc Munro
--	License: BSD
--
-- $Id: views.sql,v 1.2 2006/10/04 15:50:09 bloodnok Exp $
------------
\echo - Creating views...


\echo - ...privileges...
create view privileges(
       privilege_id,
       privilege_name) as
select privilege_id,
       privilege_name
from   vdemo_owner.privileges
where  i_have_global_priv(10001);

create rule ii_privileges as
on insert to privileges
do instead 
    insert into vdemo_owner.privileges
	   (privilege_id, privilege_name)
    select new.privilege_id, new.privilege_name 
    where  i_have_global_priv(10002);

create rule iu_privileges as
on update to privileges
do instead
    update vdemo_owner.privileges
    set    privilege_name = new.privilege_name,
	   privilege_id = new.privilege_id
    where  privilege_id = old.privilege_id
    and    i_have_global_priv(10003);
 
create rule id_privileges as
on delete to privileges
do instead
    delete from vdemo_owner.privileges
    where  privilege_id = old.privilege_id
    and    i_have_global_priv(10004);

grant select, insert, update, delete on privileges to public;    

\echo - ...roles...
create view roles(
       role_id,
       role_name) as
select role_id,
       role_name
from   vdemo_owner.roles
where  i_have_global_priv(10005);

create rule ii_roles as
on insert to roles
do instead 
    insert into vdemo_owner.roles
	   (role_id, role_name)
    select new.role_id, new.role_name
    where  i_have_global_priv(10006);

create rule iu_roles as
on update to roles
do instead
    update vdemo_owner.roles
    set    role_name = new.role_name,
	   role_id = new.role_id
    where  role_id = old.role_id
    and    i_have_global_priv(10007);
 
create rule id_roles as
on delete to roles
do instead
    delete from vdemo_owner.roles
    where  role_id = old.role_id
    and    i_have_global_priv(10008);

grant select, insert, update, delete on roles to public;    

\echo - ...role_privileges...
create view role_privileges(
       role_id,
       privilege_id) as
select role_id,
       privilege_id
from   vdemo_owner.role_privileges
where  i_have_global_priv(10009);

create rule ii_role_privileges as
on insert to role_privileges
do instead 
    insert into vdemo_owner.role_privileges
	   (role_id, privilege_id)
    select new.role_id, new.privilege_id
    where  i_have_global_priv(10010);

create rule iu_role_privileges as
on update to role_privileges
do instead
    update vdemo_owner.role_privileges
    set    role_id = new.role_id, 
	   privilege_id = new.privilege_id
    where  role_id = old.role_id
    and    privilege_id = old.privilege_id
    and    i_have_global_priv(10011);
 
create rule id_role_privileges as
on delete to role_privileges
do instead
    delete from vdemo_owner.role_privileges
    where  role_id = old.role_id
    and    i_have_global_priv(10012);

grant select, insert, update, delete on role_privileges to public;    


\echo - ...role_roles...
create view role_roles(
       role_id,
       has_role_id) as
select role_id,
       has_role_id
from   vdemo_owner.role_roles
where  i_have_global_priv(10041);

create rule ii_role_roles as
on insert to role_roles
do instead 
    insert into vdemo_owner.role_roles
	   (role_id, has_role_id)
    select new.role_id, new.has_role_id
    where  i_have_global_priv(10042);

create rule iu_role_roles as
on update to role_roles
do instead
    update vdemo_owner.role_roles
    set    role_id = new.role_id,
	   has_role_id = new.has_role_id
    where  role_id = old.role_id
    and    i_have_global_priv(10043);
 
create rule id_role_roles as
on delete to role_roles
do instead
    delete from vdemo_owner.role_roles
    where  role_id = old.role_id
    and    i_have_global_priv(10044);

grant select, insert, update, delete on role_roles to public;    


\echo - ...persons...
create view persons(
       person_id,
       person_name) as
select person_id,
       person_name
from   vdemo_owner.persons
where  i_have_personal_priv(10013, person_id);

create rule ii_persons as
on insert to persons
do instead 
    insert into vdemo_owner.persons
	   (person_id, person_name)
    select new.person_id, new.person_name
    where  i_have_personal_priv(10014, new.person_id);

create rule iu_persons as
on update to persons
do instead
    update vdemo_owner.persons
    set    person_id = new.person_id,
	   person_name = new.person_name
    where  person_id = old.person_id
    and    i_have_personal_priv(10015, old.person_id);
 
create rule id_persons as
on delete to persons
do instead
    delete from vdemo_owner.persons
    where  person_id = old.person_id
    and    i_have_personal_priv(10016, old.person_id);

grant select, insert, update, delete on persons to public;    


\echo - ...projects...
create view projects(
       project_id,
       project_name) as
select project_id,
       project_name
from   vdemo_owner.projects
where  i_have_project_priv(10017, project_id);

create rule ii_projects as
on insert to projects
do instead 
    insert into vdemo_owner.projects
	   (project_id, project_name)
    select new.project_id, new.project_name
    where  i_have_project_priv(10018, new.project_id);

create rule iu_projects as
on update to projects
do instead
    update vdemo_owner.projects
    set    project_id = new.project_id,
	   project_name = new.project_name
    where  project_id = old.project_id
    and    i_have_project_priv(10019, old.project_id);
 
create rule id_projects as
on delete to projects
do instead
    delete from vdemo_owner.projects
    where  project_id = old.project_id
    and    i_have_project_priv(10020, old.project_id);

grant select, insert, update, delete on projects to public;    


\echo - ...detail_types...
create view detail_types (
       detail_type_id,
       required_privilege_id,
       detail_type_name) as 
select detail_type_id,
       required_privilege_id,
       detail_type_name
from   vdemo_owner.detail_types
where  i_have_global_priv(10021);

create rule ii_detail_types as
on insert to detail_types
do instead 
    insert into vdemo_owner.detail_types
	   (detail_type_id, detail_type_name)
    select new.detail_type_id, new.detail_type_name
    where  i_have_global_priv(10022);

create rule iu_detail_types as
on update to detail_types
do instead
    update vdemo_owner.detail_types
    set    detail_type_id = new.detail_type_id, 
	   detail_type_name = new.detail_type_name
    where  detail_type_id = old.detail_type_id
    and    i_have_global_priv(10023);
 
create rule id_detail_types as
on delete to detail_types
do instead
    delete from vdemo_owner.detail_types
    where  detail_type_id = old.detail_type_id
    and    i_have_global_priv(10024);

grant select, insert, update, delete on detail_types to public;    


\echo - ...assignments...
create view assignments (
       project_id,
       person_id,
       role_id) as
select project_id,
       person_id,
       role_id
from   vdemo_owner.assignments
where  i_have_proj_or_pers_priv(10025, project_id, person_id);

create rule ii_assignments as
on insert to assignments
do instead 
    insert into vdemo_owner.assignments
	   (project_id, person_id, role_id)
    select new.project_id, new.person_id, new.role_id
    where  i_have_proj_or_pers_priv(10026, new.project_id, new.person_id);

create rule iu_assignments as
on update to assignments
do instead
    update vdemo_owner.assignments
    set    project_id = new.project_id,
	   person_id = new.person_id, 
	   role_id = new.person_id
    where  project_id = old.project_id
    and    person_id = old.person_id
    and    i_have_proj_or_pers_priv(10027, old.project_id, old.person_id);
 
create rule id_assignments as
on delete to assignments
do instead
    delete from vdemo_owner.assignments
    where  project_id = old.project_id
    and    person_id = old.person_id
    and    i_have_proj_or_pers_priv(10028, old.project_id, old.person_id);

grant select, insert, update, delete on assignments to public;    


\echo - ...person_roles...
create view person_roles (
       person_id,
       role_id) as
select person_id,
       role_id
from   vdemo_owner.person_roles
where  i_have_personal_priv(10029, person_id);

create rule ii_person_roles as
on insert to person_roles
do instead 
    insert into vdemo_owner.person_roles
	   (person_id, role_id)
    select new.person_id, new.role_id
    where  i_have_personal_priv(10030, new.person_id);

create rule iu_person_roles as
on update to person_roles
do instead
    update vdemo_owner.person_roles
    set    person_id = new.person_id,
	   role_id = new.role_id
    where  person_id = old.person_id
    and    role_id = old.role_id
    and    i_have_personal_priv(10031, old.person_id);
 
create rule id_person_roles as
on delete to person_roles
do instead
    delete from vdemo_owner.person_roles
    where  person_id = old.person_id
    and    role_id = old.role_id
    and    i_have_personal_priv(10032, old.person_id);

grant select, insert, update, delete on person_roles to public;    


\echo - ...project_details...
create view project_details (
       project_id,
       detail_type_id,
       value) as
select project_id,
       detail_type_id,
       value
from   vdemo_owner.project_details
where  i_have_project_priv(10033, project_id)
and    i_have_project_detail_priv(detail_type_id, project_id);

create rule ii_project_details as
on insert to project_details
do instead 
    insert into vdemo_owner.project_details
	   (project_id, detail_type_id, value)
    select new.project_id, new.detail_type_id, new.value
    where  i_have_project_priv(10034, new.project_id)
    and    i_have_project_detail_priv(new.detail_type_id, new.project_id);

create rule iu_project_details as
on update to project_details
do instead
    update vdemo_owner.project_details
    set    project_id = new.project_id, 
	   detail_type_id = new.detail_type_id, 
	   value = new.value
    where  project_id = old.project_id
    and    detail_type_id = old.detail_type_id
    and    i_have_project_priv(10035, old.project_id)
    and    i_have_project_detail_priv(old.detail_type_id, old.project_id);
 
create rule id_project_details as
on delete to project_details
do instead
    delete from vdemo_owner.project_details
    where  project_id = old.project_id
    and    detail_type_id = old.detail_type_id
    and    i_have_project_priv(10036, old.project_id)
    and    i_have_project_detail_priv(old.detail_type_id, old.project_id);

grant select, insert, update, delete on project_details to public;    


\echo - ...person_details...
create view person_details (
       person_id,
       detail_type_id,
       value) as
select person_id,
       detail_type_id,
       value
from   vdemo_owner.person_details
where  i_have_personal_priv(10037, person_id)
and    i_have_person_detail_priv(detail_type_id, person_id);

create rule ii_person_details as
on insert to person_details
do instead 
    insert into vdemo_owner.person_details
	   (person_id, detail_type_id, value)
    select new.person_id, new.detail_type_id, new.value
    where  i_have_personal_priv(10038, new.person_id)
    and    i_have_person_detail_priv(new.detail_type_id, new.person_id);

create rule iu_person_details as
on update to person_details
do instead
    update vdemo_owner.person_details
    set    person_id = new.person_id, 
	   detail_type_id = new.detail_type_id, 
	   value = new.value
    where  person_id = old.person_id
    and    detail_type_id = old.detail_type_id
    and    i_have_personal_priv(10039, old.person_id)
    and    i_have_person_detail_priv(old.detail_type_id, old.person_id);
 
create rule id_person_details as
on delete to person_details
do instead
    delete from vdemo_owner.person_details
    where  person_id = old.person_id
    and    detail_type_id = old.detail_type_id
    and    i_have_personal_priv(10040, old.person_id)
    and    i_have_person_detail_priv(old.detail_type_id, old.person_id);


grant select, insert, update, delete on person_details to public;    

\echo - ...my_global_privs...

create or replace
function global_privs_qry() returns setof int4 as '
select * from veil_bitmap_bits(''global_context'');
' language sql
stable security definer;

create or replace view my_global_privs (
       privilege_id) as
select * from global_privs_qry();

grant select on my_global_privs to public;    


\echo - ...my_personal_privs...
create or replace
function personal_privs_qry() returns setof int4 as '
select * from veil_bitmap_array_bits(''role_privs'', 11002);
' language sql
stable security definer;

create or replace view my_personal_privs (
       privilege_id) as
select * from personal_privs_qry();

grant select on my_personal_privs to public;    


\echo - ...my_projects...
create or replace
function projects_qry() returns setof varchar as '
select * from veil_bitmap_hash_entries(''project_context'');
' language sql
stable security definer;

create or replace view my_projects (
       project_id) as
select * from projects_qry();

grant select on my_projects to public;    


create type vdemo_owner.proj_privs as (
    project_id	 int4,
    privilege_id int4
);

create or replace
function project_privs() returns setof vdemo_owner.proj_privs as '
declare
    _project record;
    _priv    record;
    _result  vdemo_owner.proj_privs;
begin
    for _project in 
        select * from veil_bitmap_hash_entries(''project_context'')
    loop
        _result.project_id = _project.veil_bitmap_hash_entries;
        for _priv in
            select * from veil_bitmap_hash_bits(''project_context'', 
                                                _result.project_id)
        loop
            _result.privilege_id = _priv.veil_bitmap_hash_bits;
            return next _result;
        end loop;
    end loop;
    return;
end;
' language plpgsql
stable security definer;

grant execute on function project_privs() to public;

\echo - ...my_project_privs...
create or replace view my_project_privs (
       project_id,
       privilege_id) as
select * from project_privs();

grant select on my_project_privs to public;    


\echo - ...my_privs...
create or replace view my_privs (
    context,
    project,
    privilege_id,
    privilege_name)
as 
select a.context, a.project, a.id, p.privilege_name
from   (
        select 'Project' as context, project_id as project,
               privilege_id as id, 3 as seq
        from   my_project_privs
        union all
        select 'Global', null, privilege_id, 1
        from   my_global_privs
        union all
        select 'Personal', null, privilege_id, 2
        from   my_personal_privs
       ) a,
       privileges p
where  p.privilege_id = a.id
order by a.seq, a.context, a.project, a.id;

grant select on my_privs to public;    
