create or replace 
function test() returns boolean as
$$
declare
  _person_id int4;
  connection_id int4;
  dummy int4;
begin
  _person_id := 249305;
  select into connection_id
	      veil_int4_get('person_id');
  if connection_id != null and connection_id != _person_id then
		RAISE NOTICE 'already connected';
    return false;
  end if;

  -- Test whether the person exists
  select into dummy 1 
  from private.personid
  where person_id = _person_id;

  if found then
    RAISE NOTICE 'found';

    -- from the persons roles set the global_context bitmap.
    select into dummy
		count(veil_union_from_bitmap_array('global_context',
				'role_privs',role_id))
    from private.veil_user_roles
    where person_id = _person_id;
    RAISE NOTICE 'LOADED GLOBAL CONTEXT PRIVS';
  end if;
  return true;
end;
$$
language plpgsql stable security definer;

CREATE or replace FUNCTION veil_init(boolean) RETURNS boolean
    AS $_$
declare
	doing_reset alias for $1;
	exists_privs_range bool;
	exists_roles_range bool;
	exists_role_privs bool;
	init_reqd bool;
	dummy bool;
	dummy2 bool;
	dummy3 bool;
	dummyint int4;
	_count int4;
begin
	-- Declare all shared variables.
raise notice 'Veil_init.  doing_reset = %', doing_reset;

	select into exists_privs_range, exists_roles_range, exists_role_privs
		veil_share('privs_range'), veil_share('roles_range'),
		veil_share('role_privs');

	init_reqd = not (exists_privs_range and exists_roles_range and 
		exists_role_privs);

raise notice 'Veil_init.  exists_privs_range = %', exists_privs_range;
raise notice 'Veil_init.  exists_roles_range = %', exists_roles_range;
raise notice 'Veil_init.  exists_role_privs = %', exists_role_privs;
raise notice 'Veil_init.  init_reqd = %', init_reqd;
	if init_reqd or doing_reset then
		-- Load ranges for privs and roles.
		select into dummyint
			veil_init_range('roles_range',min(cp_id),max(cp_id))
			from private.veil_roles;

		select into dummyint
			veil_init_range('privs_range',min(cp_id),max(cp_id))
			from private.veil_privileges;

		-- initialize role_privs bitmap array
		select into dummy
			veil_init_bitmap_array('role_privs','roles_range','privs_range');

		-- populate role_privs bitmap_array
		select into _count
			count(veil_bitmap_array_setbit('role_privs',role_id,privilege_id))
			from private.veil_role_privileges;

	end if;

	-- Declare important session variables, so that we do not get odd,
	-- undefined variable error messages.
	select into dummyint, dummy, dummy2, dummy3
		 veil_int4_set('person_id', null),
		 veil_init_bitmap('global_context', 'privs_range'),
		 veil_init_bitmap_hash('location_context', 'privs_range'),
		 veil_init_bitmap_hash('account_code_context', 'privs_range');

	return true;
end;
$_$
LANGUAGE plpgsql STABLE SECURITY DEFINER;
